// DHT Exporter for Prometheus

package main

import (
    "gitfox.ru/victor/go-dht"
    "github.com/prometheus/client_golang/prometheus"
    "github.com/prometheus/client_golang/prometheus/promhttp"
    "os"
    "net/http"
    "fmt"
    "time"
    "flag"
)

var (
    m_temp = prometheus.NewGauge(
        prometheus.GaugeOpts{
            Name: "dht_sensor_temperature_total",
            Help: "DHT Sensor Temperature",
        },
    )

    m_hum = prometheus.NewGauge(
        prometheus.GaugeOpts{
            Name: "dht_sensor_humidity_total",
            Help: "DHT Sensor Humidity",
        },
    )

    address string
    pin int
    interval int
    file string

    cached_temp, cached_hum float64
)

func init() {
    flag.StringVar(&address, "listen-address", ":4554", "The address to listen on for HTTP requests")
    flag.IntVar(&pin, "pin", 7, "Pin number for DHT sensor")
    flag.IntVar(&interval, "interval", 3, "Sensor read interval in seconds")
    flag.StringVar(&file, "data-file", "", "Save last DHT sensor data to a file")
}

func main() {
    fmt.Printf("DHT Exporter, v1.0\n")

    flag.Parse()
    if interval < 1 {
        fmt.Printf("WARNING: Invalid interval! Using default value.\n")
        interval = 3
    }

    /* Registering prometheus metrics */
    prometheus.MustRegister(m_temp, m_hum)
    go func() {
        for true {
            temp, hum, err := dht.ReadDHT(pin)
            if err == nil {
                if temp >= 0 && temp < 125 {
                    cached_temp = temp
                }
                if hum >= 0 && temp <= 100 {
                    cached_hum = hum
                }
            }
            m_temp.Set(cached_temp)
            m_hum.Set(cached_hum)
            if file != "" {
                content := fmt.Sprintf("%.1f %.1f", cached_temp, cached_hum)
                err = os.WriteFile(file, []byte(content), 0666)
            }
            time.Sleep(time.Duration(interval * 1000) * time.Millisecond)
        }
    }()

    /* Starting HTTP server */
    fmt.Println("Starting HTTP server...")
    fmt.Printf("  - server port    %s\n", address)
    fmt.Printf("  - sensor pin     %d\n", pin)
    fmt.Printf("  - read interval  %d\n", interval)
    if (file != "") {
        fmt.Printf("  - save to file   %s\n", file)
    }
    http.Handle("/metrics", promhttp.Handler())
    err := http.ListenAndServe(address, nil)
    if err != nil {
        fmt.Println("Failed:", err)
    }
}
